///////////////////////////////////////////////////////////////////////////////
// Filename: userinterfaceclass.cpp
///////////////////////////////////////////////////////////////////////////////
#include "userinterfaceclass.h"


UserInterfaceClass::UserInterfaceClass()
{
	m_Font = 0;
	m_sentence1 = 0;
	m_sentence2 = 0;
	m_sentence3 = 0;
}


UserInterfaceClass::UserInterfaceClass(const UserInterfaceClass& other)
{
}


UserInterfaceClass::~UserInterfaceClass()
{
}


bool UserInterfaceClass::Initialize(ID3D10Device* device, HWND hwnd, int screenWidth, int screenHeight, D3DXMATRIX baseViewMatrix)
{
	bool result;


	// Store the size of the screen.
	m_screenWidth = screenWidth;
	m_screenHeight = screenHeight;

	// Create the font object.
	m_Font = new FontClass;
	if(!m_Font)
	{
		return false;
	}

	// Initialize the font object.
	result = m_Font->Initialize(device, hwnd, baseViewMatrix);
	if(!result)
	{
		return false;
	}

	// Create a new sentence object.
	m_sentence1 = new SentenceType;
	if(!m_sentence1)
	{
		return false;
	}

	// Initialize the sentence buffer.
	result = InitializeSentence(m_sentence1, 16, device);
	if(!result)
	{
		return false;
	}

	// Now update the sentence vertex buffer with the new string information.
	result = UpdateSentence(m_sentence1, "Fps: 0", 20, 20, 0.0f, 1.0f, 0.0f);
	if(!result)
	{
		return false;
	}

	// Create a new sentence object.
	m_sentence2 = new SentenceType;
	if(!m_sentence2)
	{
		return false;
	}

	// Initialize the sentence buffer.
	result = InitializeSentence(m_sentence2, 16, device);
	if(!result)
	{
		return false;
	}

	// Now update the sentence vertex buffer with the new string information.
	result = UpdateSentence(m_sentence2, "Cpu: 0%", 20, 40, 0.0f, 1.0f, 0.0f);
	if(!result)
	{
		return false;
	}

	// Create a new sentence object.
	m_sentence3 = new SentenceType;
	if(!m_sentence3)
	{
		return false;
	}

	// Initialize the sentence buffer.
	result = InitializeSentence(m_sentence3, 32, device);
	if(!result)
	{
		return false;
	}

	// Now update the sentence vertex buffer with the new string information.
	result = UpdateSentence(m_sentence3, "Latency: 0 ms", 20, 60, 0.0f, 1.0f, 0.0f);
	if(!result)
	{
		return false;
	}

	return true;
}


void UserInterfaceClass::Shutdown()
{
	// Release the first sentence.
	ShutdownSentence(m_sentence1);

	// Release the second sentence.
	ShutdownSentence(m_sentence2);

	// Release the third sentence.
	ShutdownSentence(m_sentence3);

	// Release the font object.
	if(m_Font)
	{
		m_Font->Shutdown();
		delete m_Font;
		m_Font = 0;
	}

	return;
}


void UserInterfaceClass::Render(ID3D10Device* device)
{
	// Draw the first sentence.
	RenderSentence(device, m_sentence1);
	
	// Draw the second sentence.
	RenderSentence(device, m_sentence2);

	// Draw the third sentence.
	RenderSentence(device, m_sentence3);

	return;
}


void UserInterfaceClass::SetMatrices(D3DXMATRIX worldMatrix, D3DXMATRIX orthoMatrix)
{
	// Set the matrices inside the font shader object.
	m_Font->SetMatrices(worldMatrix, orthoMatrix);

	return;
}


bool UserInterfaceClass::InitializeSentence(SentenceType* sentence, int maxLength, ID3D10Device* device)
{
	VertexType* vertices;
	unsigned long* indices;
	int i;
	D3D10_BUFFER_DESC vertexBufferDesc;
    D3D10_SUBRESOURCE_DATA vertexData;
	HRESULT result;
	D3D10_BUFFER_DESC indexBufferDesc;
    D3D10_SUBRESOURCE_DATA indexData;


	// Initialize the pointers to null in the sentence structure.
	sentence->vertexBuffer = 0;
	sentence->indexBuffer = 0;

	// Set the maximum length of the sentence.
	sentence->maxLength = maxLength;

	// Set the number of vertices in the vertex array.
	sentence->vertexCount = 6 * sentence->maxLength;

	// Create the vertex array.
	vertices = new VertexType[sentence->vertexCount];
	if(!vertices)
	{
		return false;
	}

	// Set the number of indexes in the index array.
	sentence->indexCount = sentence->vertexCount;

	// Create the index array.
	indices = new unsigned long[sentence->indexCount];
	if(!indices)
	{
		return false;
	}

	// Initialize vertex array to zeros at first.
	memset(vertices, 0, (sizeof(VertexType) * sentence->vertexCount));

	// Initialize the index array.
	for(i=0; i<sentence->indexCount; i++)
	{
		indices[i] = i;
	}

	// Set up the description of the dynamic vertex buffer.
    vertexBufferDesc.Usage = D3D10_USAGE_DYNAMIC;
    vertexBufferDesc.ByteWidth = sizeof(VertexType) * sentence->vertexCount;
    vertexBufferDesc.BindFlags = D3D10_BIND_VERTEX_BUFFER;
    vertexBufferDesc.CPUAccessFlags = D3D10_CPU_ACCESS_WRITE;
    vertexBufferDesc.MiscFlags = 0;

	// Give the subresource structure a pointer to the vertex data.
    vertexData.pSysMem = vertices;

	// Create the vertex buffer.
    result = device->CreateBuffer(&vertexBufferDesc, &vertexData, &sentence->vertexBuffer);
	if(FAILED(result))
	{
		return false;
	}

	// Set up the description of the static index buffer.
    indexBufferDesc.Usage = D3D10_USAGE_DEFAULT;
    indexBufferDesc.ByteWidth = sizeof(unsigned long) * sentence->indexCount;
    indexBufferDesc.BindFlags = D3D10_BIND_INDEX_BUFFER;
    indexBufferDesc.CPUAccessFlags = 0;
    indexBufferDesc.MiscFlags = 0;

	// Give the subresource structure a pointer to the index data.
    indexData.pSysMem = indices;

	// Create the index buffer.
	result = device->CreateBuffer(&indexBufferDesc, &indexData, &sentence->indexBuffer);
	if(FAILED(result))
	{
		return false;
	}

	// Release the vertex array as it is no longer needed.
	delete [] vertices;
	vertices = 0;

	// Release the index array as it is no longer needed.
	delete [] indices;
	indices = 0;

	return true;
}


void UserInterfaceClass::ShutdownSentence(SentenceType* sentence)
{
	// Release the vertex buffer.
	if(sentence->vertexBuffer)
	{
		sentence->vertexBuffer->Release();
		sentence->vertexBuffer = 0;
	}

	// Release the index buffer.
	if(sentence->indexBuffer)
	{
		sentence->indexBuffer->Release();
		sentence->indexBuffer = 0;
	}

	return;
}


bool UserInterfaceClass::UpdateSentence(SentenceType* sentence, char* text, int positionX, int positionY, 
										float red, float green, float blue)
{
	int numLetters;
	VertexType* vertices;
	float drawX, drawY;
	HRESULT result;
	void* verticesPtr;


	// Store the color of the sentence.
	sentence->red = red;
	sentence->green = green;
	sentence->blue = blue;

	// Get the number of letters in the sentence.
	numLetters = (int)strlen(text);

	// Check for possible buffer overflow.
	if(numLetters > sentence->maxLength)
	{
		return false;
	}

	// Create the vertex array.
	vertices = new VertexType[sentence->vertexCount];
	if(!vertices)
	{
		return false;
	}

	// Initialize vertex array to zeros at first.
	memset(vertices, 0, (sizeof(VertexType) * sentence->vertexCount));

	// Calculate the X and Y pixel position on the screen to start drawing to.
	drawX = (float)(((m_screenWidth / 2) * -1) + positionX);
	drawY = (float)((m_screenHeight / 2) - positionY);

	// Use the font class to build the vertex array from the sentence text and sentence draw location.
	m_Font->BuildVertexArray((void*)vertices, text, drawX, drawY);

	// Initialize the vertex buffer pointer to null first.
	verticesPtr = 0;

	// Lock the vertex buffer.
	result = sentence->vertexBuffer->Map(D3D10_MAP_WRITE_DISCARD, 0, (void**)&verticesPtr);
	if(FAILED(result))
	{
		return false;
	}

	// Copy the vertex array into the vertex buffer.
	memcpy(verticesPtr, (void*)vertices, (sizeof(VertexType) * sentence->vertexCount));

	// Unlock the vertex buffer.
	sentence->vertexBuffer->Unmap();

	// Release the vertex array as it is no longer needed.
	delete [] vertices;
	vertices = 0;

	return true;
}


void UserInterfaceClass::RenderSentence(ID3D10Device* device, SentenceType* sentence)
{
	D3DXVECTOR4 pixelColor;


	// Put the vertex buffer on the graphics pipeline to prepare it for drawing.
	RenderBuffers(device, sentence);

	// Create a pixel color vector with the input sentence color.
	pixelColor = D3DXVECTOR4(sentence->red, sentence->green, sentence->blue, 1.0f);

	// Set the pixel color in the font shader.
	m_Font->SetShaderPixelColor(pixelColor);

	// Now render the prepared vertex buffer with the shader.
	m_Font->RenderShader(device, sentence->indexCount);

	return;
}


void UserInterfaceClass::RenderBuffers(ID3D10Device* device, SentenceType* sentence)
{
	unsigned int stride;
	unsigned int offset;


	// Set vertex buffer
    stride = sizeof(VertexType); 
	offset = 0;
    
	// Set the vertex buffer to active in the input assembler so it can be rendered.
	device->IASetVertexBuffers(0, 1, &sentence->vertexBuffer, &stride, &offset);

    // Set the index buffer to active in the input assembler so it can be rendered.
    device->IASetIndexBuffer(sentence->indexBuffer, DXGI_FORMAT_R32_UINT, 0);

    // Set the type of primitive that should be rendered from this vertex buffer, in this case triangles.
    device->IASetPrimitiveTopology(D3D10_PRIMITIVE_TOPOLOGY_TRIANGLELIST);

	return;
}


bool UserInterfaceClass::SetFps(int fps)
{
	char tempString[16];
	char fpsString[16];
	float red, green, blue;
	bool result;


	// Truncate the fps to below 10,000.
	if(fps > 9999)
	{
		fps = 9999;
	}

	// Convert the fps integer to a string format.
	_itoa_s(fps, tempString, 10);

	// Setup the fps string.
	strcpy_s(fpsString, "Fps: ");
	strcat_s(fpsString, tempString);

	// If fps is 60 or above set the fps color to green.
	if(fps >= 60)
	{
		red = 0.0f;
		green = 1.0f;
		blue = 0.0f;
	}

	// If fps is below 60 set the fps color to yellow.
	if(fps < 60)
	{
		red = 1.0f;
		green = 1.0f;
		blue = 0.0f;
	}

	// If fps is below 30 set the fps color to red.
	if(fps < 30)
	{
		red = 1.0f;
		green = 0.0f;
		blue = 0.0f;
	}

	// Now update the sentence vertex buffer with the new string information.
	result = UpdateSentence(m_sentence1, fpsString, 20, 20, red, green, blue);
	if(!result)
	{
		return false;
	}

	return true;
}


bool UserInterfaceClass::SetCpu(int cpu)
{
	char tempString[16];
	char cpuString[16];
	bool result;


	// Convert the cpu interger to string format.
	_itoa_s(cpu, tempString, 10);

	// Setup the cpu string.
	strcpy_s(cpuString, "Cpu: ");
	strcat_s(cpuString, tempString);
	strcat_s(cpuString, "%");

	// Update the sentence vertex buffer with the new string information.
	result = UpdateSentence(m_sentence2, cpuString, 20, 40, 0.0f, 1.0f, 0.0f);
	if(!result)
	{
		return false;
	}

	return true;
}


bool UserInterfaceClass::SetLatency(int latency)
{
	char tempString[16];
	char latencyString[16];
	bool result;


	// Truncate the latency to below 10,000.
	if(latency > 9999)
	{
		latency = 9999;
	}

	// Convert the latency integer to string format.
	_itoa_s(latency, tempString, 10);

	// Setup the cpu string.
	strcpy_s(latencyString, "Latency: ");
	strcat_s(latencyString, tempString);
	strcat_s(latencyString, " ms");

	// Update the sentence vertex buffer with the new string information.
	result = UpdateSentence(m_sentence3, latencyString, 20, 60, 0.0f, 1.0f, 0.0f);
	if(!result)
	{
		return false;
	}

	return true;
}