////////////////////////////////////////////////////////////////////////////////
// Filename: networkclass.cpp
////////////////////////////////////////////////////////////////////////////////
#include "networkclass.h"


NetworkClass::NetworkClass()
{
  m_clientList = 0;
}


NetworkClass::NetworkClass(const NetworkClass& other)
{
}


NetworkClass::~NetworkClass()
{
}


bool NetworkClass::Initialize()
{
  bool result;


  // Initialize the client list.
  result = InitializeClientList();
  if(!result)
  {
    return false;
  }

  // Initialize and start the server socket to listen and process incoming connections.
  result = InitializeServerSocket();
  if(!result)
  {
    return false;
  }

  return true;
}


void NetworkClass::Shutdown()
{
  // Release the client list.
  ShutdownClientList();

  // Close and release the server socket.
  ShutdownServerSocket();

  return;
}


bool NetworkClass::InitializeClientList()
{
  int i;


  // Create the client list.
  m_clientList = new ClientType[MAX_CLIENTS];
  if(!m_clientList)
  {
    return false;
  }

  // Loop through all the clients and initialize them.
  for(i=0; i<MAX_CLIENTS; i++)
  {
    m_clientList[i].online = false;
  }

  return true;
}


void NetworkClass::ShutdownClientList()
{
  int i;


  // Let each client know it has been disconnected from the server.
  for(i=0; i<MAX_CLIENTS; i++)
  {

  }

  // Then delete the client list.
  if(m_clientList)
  {
    delete [] m_clientList;
    m_clientList = 0;
  }

  return;
}


bool NetworkClass::InitializeServerSocket()
{  
  struct sockaddr_in serverAddress;
  int error;
  unsigned long setting;
  pthread_t serverThreadId;


  // Set the server to be online.
  m_online = true;

  // Create a UDP socket.
  m_socket = socket(AF_INET, SOCK_DGRAM, 0);
  if(m_socket == -1)
  {
    cout << "Error: Could not create UDP socket." << endl;
    return false;
  }

  // Fill in the address information for binding the socket to and have the kernel set the IP address.
  memset((char*)&serverAddress, 0, sizeof(serverAddress));
  serverAddress.sin_family = AF_INET;
  serverAddress.sin_port = htons(PORT_NUMBER);
  serverAddress.sin_addr.s_addr = htonl(INADDR_ANY);

  // Bind the socket to the address.
  error = bind(m_socket, (struct sockaddr*)&serverAddress, sizeof(serverAddress));
  if(error == -1)
  {
    cout << "Error: Could not bind the socket." << endl;
    return false;
  }

  // Set the socket to non-blocking I/O.
  setting = 1;
  
  error = ioctl(m_socket, FIONBIO, &setting);
  if(error == -1)
  {
    cout << "Error: Could not set socket to non-blocking I/O." << endl;
    return false;
  }

  // Create a thread to listen for and accept incoming connections from clients.
  error = pthread_create(&serverThreadId, NULL, ServerListenFunction, (void*)this);
  if(error != 0)
  {
    cout << "Error: Could not create thread." << endl;
    return false;
  }

  return true;
}


void NetworkClass::ShutdownServerSocket()
{
  int error;


  // Close the server socket.
  error = close(m_socket);
  if(error != 0)
  {
    cout << "Error: Could not close socket correctly." << endl;
  }

  return;
}


bool NetworkClass::Online()
{
  return m_online;
}


int NetworkClass::GetServerSocket()
{
  return m_socket;
}


void* ServerListenFunction(void* ptr)
{
  NetworkClass* networkClassPtr;
  int bytesRead;
  char recvBuffer[4096];
  struct sockaddr_in clientAddress;
  unsigned int clientLength;


  // Get a pointer to the calling object.
  networkClassPtr = (NetworkClass*)ptr;

  // Set the size of the address.
  clientLength = sizeof(clientAddress);

  while(networkClassPtr->Online())
  {
    // Check if there is a message from a client.
    bytesRead = recvfrom(networkClassPtr->GetServerSocket(), recvBuffer, 4096, 0, (struct sockaddr*)&clientAddress, &clientLength);

    if(bytesRead > 0)
    {
      networkClassPtr->ReadNetworkMessage(recvBuffer, clientAddress);
    }

  }
  
  return 0;
}


void NetworkClass::ReadNetworkMessage(char* recvBuffer, struct sockaddr_in clientAddress)
{
  char* ipAddress;
  MSG_GENERIC_DATA* message;


  // Store the IP address of the person who sent a message.
  ipAddress = inet_ntoa(clientAddress.sin_addr);

  // Coerce the message into a generic format to read the type of message.
  message = (MSG_GENERIC_DATA*)recvBuffer;

  switch(message->type)
  {
    case MSG_CONNECT:
    {
      cout << ipAddress << " sent a connect message." << endl;
      HandleConnectMessage(clientAddress);
      break;
    }
    case MSG_PING:
    {
      HandlePingMessage(message->fromId);
      break;
    }
    case MSG_DISCONNECT:
    {
      HandleDisconnectMessage(message->fromId);
      break;
    }
    default:
    {
      break;
    }
  }

  return;
}


void NetworkClass::HandleConnectMessage(struct sockaddr_in clientAddress)
{
  short newId;
  MSG_GENERIC_DATA message;
  int bytesSent;


  // Get the next free ID number and assign it to this client.
  newId = GetNextIdNumber();

  if(newId != -1)
  {
    // Store the client address information so that we can send them messages when we need to.
    m_clientList[newId].clientAddress = clientAddress;
  }

  message.type = MSG_ID;
  message.toId = newId;
  message.fromId = ID_SERVER;

  bytesSent = sendto(m_socket, (char*)&message, sizeof(MSG_GENERIC_DATA), 0, (struct sockaddr*)&clientAddress, sizeof(clientAddress));
  if(bytesSent != sizeof(MSG_GENERIC_DATA))
  {
    cout << "Error: Could not send ID message." << endl;
  }
  else
  {
    cout << "Sent ID number " << newId << " to client." << endl;
  }
  
  return;
}


short NetworkClass::GetNextIdNumber()
{
  short idNumber;
  bool done;
  int i;


  // Initialize the loop variables.
  idNumber = -1;
  done = false;
  i=0;

  // Loop through all the clients and assign an ID that is not online.
  while(!done)
  {
    if(m_clientList[i].online == false)
    {
      idNumber = i;
      m_clientList[idNumber].online = true;
      done = true;
    }
    else
    {
      i++;
    }

    // Check if we have exceeded the maximum number of clients.
    if(i == MAX_CLIENTS)
    {
      done = true;
    }
  }

  return idNumber;
}


void NetworkClass::HandlePingMessage(short fromId)
{
  MSG_GENERIC_DATA message;
  int bytesSent;


  message.type = MSG_PING;
  message.toId = fromId;
  message.fromId = ID_SERVER;

  bytesSent = sendto(m_socket, (char*)&message, sizeof(MSG_GENERIC_DATA), 0, (struct sockaddr*)&m_clientList[fromId].clientAddress, 
					 sizeof(m_clientList[fromId].clientAddress));
  if(bytesSent != sizeof(MSG_GENERIC_DATA))
  {
    cout << "Error: Could not send ping message." << endl;
  }
  else
  {
    cout << "Sent ping message to client " << fromId << "." << endl;
  }
  
  return;
}


void NetworkClass::HandleDisconnectMessage(short fromId)
{
  cout << "Disconnect meesage from " << fromId << ", shutting server down also." << endl;

  // Set the server to be offline.
  m_online = false;
  
  return;
}
