////////////////////////////////////////////////////////////////////////////////
// Filename: networkclass.h
////////////////////////////////////////////////////////////////////////////////
#ifndef _NETWORKCLASS_H_
#define _NETWORKCLASS_H_


/////////////
// GLOBALS //
/////////////
const int MAX_CLIENTS = 1000;
const unsigned short PORT_NUMBER = 7531;


//////////////
// INCLUDES //
//////////////
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <string.h>
#include <sys/ioctl.h>
#include <pthread.h>
#include <iostream>
using namespace std;


///////////////////////
// MY CLASS INCLUDES //
///////////////////////
#include "networkmessages.h"


////////////////////////////////////////////////////////////////////////////////
// Class name: NetworkClass
////////////////////////////////////////////////////////////////////////////////
class NetworkClass
{
private:
  struct ClientType
  {
    bool online;
    struct sockaddr_in clientAddress;
  };

public:
    NetworkClass();
    NetworkClass(const NetworkClass&);
    ~NetworkClass();

    bool Initialize();
    void Shutdown();

    bool Online();
    int GetServerSocket();
    void ReadNetworkMessage(char*, struct sockaddr_in);

private:
    bool InitializeClientList();
    void ShutdownClientList();

    bool InitializeServerSocket();
    void ShutdownServerSocket();

    void HandleConnectMessage(struct sockaddr_in);
    short GetNextIdNumber();
    void HandlePingMessage(short);
    void HandleDisconnectMessage(short);

private:
    ClientType* m_clientList;
    bool m_online;
    int m_socket;
};


/////////////////////////
// FUNCTION PROTOTYPES //
/////////////////////////
void* ServerListenFunction(void*);

#endif
